@@ -3,7 +3,7 @@ methods::setGeneric("simplex", function(data, ...) standardGeneric("simplex"))
 .simplex_sf_method = \(data,target,lib = NULL,pred = NULL,E = 1:10,tau = 1,k = E+2,
                        nb = NULL, threads = detectThreads(), trend.rm = TRUE){
   vec = .uni_lattice(data,target,trend.rm)
-  if (is.null(lib)) lib = 1:nrow(data)
+  if (is.null(lib)) lib = which(!is.na(vec))
   if (is.null(pred)) pred = lib
   if (is.null(nb)) nb = .internal_lattice_nb(data)
   res = RcppSimplex4Lattice(vec,nb,lib,pred,E,k,tau,threads)
@@ -13,7 +13,7 @@ methods::setGeneric("simplex", function(data, ...) standardGeneric("simplex"))
 .simplex_spatraster_method = \(data,target,lib = NULL,pred = NULL,E = 1:10,tau = 1,
                                k = E+2, threads = detectThreads(), trend.rm = TRUE){
   mat = .uni_grid(data,target,trend.rm)
-  if (is.null(lib)) lib = .internal_samplemat(mat)
+  if (is.null(lib)) lib = which(!is.na(mat), arr.ind = TRUE)
   if (is.null(pred)) pred = lib
   res = RcppSimplex4Grid(mat,lib,pred,E,k,tau,threads)
   return(.bind_xmapself(res,target))
@@ -24,7 +24,7 @@ methods::setGeneric("simplex", function(data, ...) standardGeneric("simplex"))
 #' @inheritParams embedded
 #' @param lib (optional) Libraries indices.
 #' @param pred (optional) Predictions indices.
-#' @param k (optional) Number of nearest neighbors used for prediction.
+#' @param k (optional) Number of nearest neighbors used in prediction.
 #' @param threads (optional) Number of threads.
 #'
 #' @return A list
@@ -37,11 +37,13 @@ methods::setGeneric("simplex", function(data, ...) standardGeneric("simplex"))
 #' @name simplex
 #' @rdname simplex
 #' @aliases simplex,sf-method
+#' @references
+#' Sugihara G. and May R. 1990. Nonlinear forecasting as a way of distinguishing chaos from measurement error in time series. Nature, 344:734-741.
 #'
 #' @examples
-#' columbus = sf::read_sf(system.file("shapes/columbus.gpkg", package="spData"))
+#' columbus = sf::read_sf(system.file("case/columbus.gpkg", package="spEDM"))
 #' \donttest{
-#' simplex(columbus,target = "CRIME")
+#' simplex(columbus,"crime")
 #' }
 methods::setMethod("simplex", "sf", .simplex_sf_method)
 
