@@ -2,32 +2,36 @@
 #define Forecast4Grid_H
 
 #include <vector>
+#include <cmath>
+#include <algorithm>
+#include <utility>
 #include "CppGridUtils.h"
 #include "SimplexProjection.h"
 #include "SMap.h"
 #include <RcppThread.h>
 
 /*
- * Evaluates prediction performance of different embedding dimensions for grid data using simplex projection.
+ * Evaluates prediction performance of different combinations of embedding dimensions and number of nearest neighbors
+ * for grid data using simplex projection.
  *
  * Parameters:
  *   - mat: A matrix to be embedded.
  *   - lib_indices: A boolean vector indicating library (training) set indices.
  *   - pred_indices: A boolean vector indicating prediction set indices.
  *   - E: A vector of embedding dimensions to evaluate.
+ *   - b: A vector of nearest neighbors to use for prediction.
  *   - tau: The spatial lag step for constructing lagged state-space vectors.
- *   - b: Number of nearest neighbors to use for prediction.
  *   - threads: Number of threads used from the global pool.
  *
  * Returns:
- *   A 2D vector where each row contains [E, rho, mae, rmse] for a given embedding dimension.
+ *   A 2D vector where each row contains [E, b, rho, mae, rmse] for a given embedding dimension.
  */
 std::vector<std::vector<double>> Simplex4Grid(const std::vector<std::vector<double>>& mat,
                                               const std::vector<bool>& lib_indices,
                                               const std::vector<bool>& pred_indices,
                                               const std::vector<int>& E,
+                                              const std::vector<int>& b,
                                               int tau,
-                                              int b,
                                               int threads);
 
 /*
