@@ -12,7 +12,6 @@
 #include "spEDMDataStruct.h"
 #include <RcppThread.h>
 
-// [[Rcpp::plugins(cpp11)]]
 // [[Rcpp::depends(RcppThread)]]
 
 /**
@@ -74,7 +73,6 @@ std::vector<double> PartialSimplex4Grid(
       rho[0] = PearsonCor(target,target_pred,true);
       rho[1] = PartialCorTrivar(target,target_pred,con_pred,true,false);
     }
-
   } else {
     std::vector<std::vector<double>> con_pred(n_controls);
     std::vector<double> temp_pred;
@@ -94,8 +92,8 @@ std::vector<double> PartialSimplex4Grid(
       rho[0] = PearsonCor(target,target_pred,true);
       rho[1] = PartialCor(target,target_pred,con_pred,true,false);
     }
-
   }
+
   return rho;
 }
 
@@ -160,7 +158,6 @@ std::vector<double> PartialSMap4Grid(
       rho[0] = PearsonCor(target,target_pred,true);
       rho[1] = PartialCorTrivar(target,target_pred,con_pred,true,false);
     }
-
   } else {
     std::vector<std::vector<double>> con_pred(n_controls);
     std::vector<double> temp_pred;
@@ -180,7 +177,6 @@ std::vector<double> PartialSMap4Grid(
       rho[0] = PearsonCor(target,target_pred,true);
       rho[1] = PartialCor(target,target_pred,con_pred,true,false);
     }
-
   }
 
   return rho;
@@ -380,7 +376,6 @@ std::vector<PartialCorRes> SCPCMSingle4GridOneDim(
     x_xmap_y.emplace_back(lib_size, rho[0], rho[1]);
     return x_xmap_y;
   } else if (parallel_level == 0) {
-
     // Precompute valid indices for the library
     std::vector<std::vector<int>> valid_lib_indices;
     for (int start_lib = 0; start_lib < max_lib_size; ++start_lib) {
@@ -438,7 +433,6 @@ std::vector<PartialCorRes> SCPCMSingle4GridOneDim(
 
     return x_xmap_y;
   } else {
-
     // Precompute valid indices for the library
     std::vector<std::vector<int>> valid_lib_indices;
     for (int start_lib = 0; start_lib < max_lib_size; ++start_lib) {
@@ -651,13 +645,13 @@ std::vector<std::vector<double>> SCPCM4Grid(
   // Set library indices
   std::vector<bool> lib_indices(totalRow * totalCol, false);
   for (const auto& l : lib) {
-    lib_indices[LocateGridIndices(l.first, l.second, totalRow, totalCol)] = true;
+    lib_indices[LocateGridIndices(l.first + 1, l.second + 1, totalRow, totalCol)] = true;
   }
 
   // Set prediction indices
   std::vector<bool> pred_indices(totalRow * totalCol, false);
   for (const auto& p : pred) {
-    pred_indices[LocateGridIndices(p.first, p.second, totalRow, totalCol)] = true;
+    pred_indices[LocateGridIndices(p.first + 1, p.second + 1, totalRow, totalCol)] = true;
   }
 
   // Exclude NA values in yPred from the library and prediction indices
@@ -668,8 +662,8 @@ std::vector<std::vector<double>> SCPCM4Grid(
     }
   }
 
-  // Initialize the result container
-  std::vector<PartialCorRes> x_xmap_y;
+  // Local results for each library
+  std::vector<std::vector<PartialCorRes>> local_results(unique_lib_size_pairs.size());
 
   if (parallel_level == 0){
     // Iterate over each library size
@@ -678,7 +672,7 @@ std::vector<std::vector<double>> SCPCM4Grid(
       for (size_t i = 0; i < unique_lib_size_pairs.size(); ++i) {
         int lib_size_row = unique_lib_size_pairs[i].first;
         int lib_size_col = unique_lib_size_pairs[i].second;
-        auto results = SCPCMSingle4Grid(
+        local_results[i] = SCPCMSingle4Grid(
           xEmbedings,
           yPred,
           zMatrixs,
@@ -696,14 +690,13 @@ std::vector<std::vector<double>> SCPCM4Grid(
           parallel_level,
           cumulate,
           row_size_mark);
-        x_xmap_y.insert(x_xmap_y.end(), results.begin(), results.end());
         bar++;
       }
     } else {
       for (size_t i = 0; i < unique_lib_size_pairs.size(); ++i) {
         int lib_size_row = unique_lib_size_pairs[i].first;
         int lib_size_col = unique_lib_size_pairs[i].second;
-        auto results = SCPCMSingle4Grid(
+        local_results[i] = SCPCMSingle4Grid(
           xEmbedings,
           yPred,
           zMatrixs,
@@ -721,7 +714,6 @@ std::vector<std::vector<double>> SCPCM4Grid(
           parallel_level,
           cumulate,
           row_size_mark);
-        x_xmap_y.insert(x_xmap_y.end(), results.begin(), results.end());
       }
     }
   } else {
@@ -731,7 +723,7 @@ std::vector<std::vector<double>> SCPCM4Grid(
       RcppThread::parallelFor(0, unique_lib_size_pairs.size(), [&](size_t i) {
         int lib_size_row = unique_lib_size_pairs[i].first;
         int lib_size_col = unique_lib_size_pairs[i].second;
-        auto results = SCPCMSingle4Grid(
+        local_results[i] = SCPCMSingle4Grid(
           xEmbedings,
           yPred,
           zMatrixs,
@@ -749,7 +741,6 @@ std::vector<std::vector<double>> SCPCM4Grid(
           parallel_level,
           cumulate,
           row_size_mark);
-        x_xmap_y.insert(x_xmap_y.end(), results.begin(), results.end());
         bar++;
       }, threads_sizet);
     } else {
@@ -757,7 +748,7 @@ std::vector<std::vector<double>> SCPCM4Grid(
       RcppThread::parallelFor(0, unique_lib_size_pairs.size(), [&](size_t i) {
         int lib_size_row = unique_lib_size_pairs[i].first;
         int lib_size_col = unique_lib_size_pairs[i].second;
-        auto results = SCPCMSingle4Grid(
+        local_results[i] = SCPCMSingle4Grid(
           xEmbedings,
           yPred,
           zMatrixs,
@@ -775,11 +766,18 @@ std::vector<std::vector<double>> SCPCM4Grid(
           parallel_level,
           cumulate,
           row_size_mark);
-        x_xmap_y.insert(x_xmap_y.end(), results.begin(), results.end());
       }, threads_sizet);
     }
   }
 
+  // Initialize the result container
+  std::vector<PartialCorRes> x_xmap_y;
+
+  // Merge all local results into the final result
+  for (const auto& local_result : local_results) {
+    x_xmap_y.insert(x_xmap_y.end(), local_result.begin(), local_result.end());
+  }
+
   // Group by the first int and store second and third values as pairs
   std::map<int, std::vector<std::pair<double, double>>> grouped_results;
 
@@ -953,19 +951,19 @@ std::vector<std::vector<double>> SCPCM4GridOneDim(
   std::sort(unique_lib_sizes.begin(), unique_lib_sizes.end());
   unique_lib_sizes.erase(std::unique(unique_lib_sizes.begin(), unique_lib_sizes.end()), unique_lib_sizes.end());
 
-  // Initialize the result container
-  std::vector<PartialCorRes> x_xmap_y;
+  // Local results for each library
+  std::vector<std::vector<PartialCorRes>> local_results(unique_lib_sizes.size());
 
   if (parallel_level == 0){
     // Iterate over each library size
     if (progressbar) {
       RcppThread::ProgressBar bar(unique_lib_sizes.size(), 1);
-      for (int lib_size : unique_lib_sizes) {
-        auto results = SCPCMSingle4GridOneDim(
+      for (size_t i = 0; i < unique_lib_sizes.size(); ++i) {
+        local_results[i] = SCPCMSingle4GridOneDim(
           xEmbedings,
           yPred,
           zMatrixs,
-          lib_size,
+          unique_lib_sizes[i],
           max_lib_size,
           possible_lib_indices,
           pred_indices,
@@ -980,16 +978,15 @@ std::vector<std::vector<double>> SCPCM4GridOneDim(
           parallel_level,
           cumulate
         );
-        x_xmap_y.insert(x_xmap_y.end(), results.begin(), results.end());
         bar++;
       }
     } else {
-      for (int lib_size : unique_lib_sizes) {
-        auto results = SCPCMSingle4GridOneDim(
+      for (size_t i = 0; i < unique_lib_sizes.size(); ++i) {
+        local_results[i] = SCPCMSingle4GridOneDim(
           xEmbedings,
           yPred,
           zMatrixs,
-          lib_size,
+          unique_lib_sizes[i],
           max_lib_size,
           possible_lib_indices,
           pred_indices,
@@ -1004,7 +1001,6 @@ std::vector<std::vector<double>> SCPCM4GridOneDim(
           parallel_level,
           cumulate
         );
-        x_xmap_y.insert(x_xmap_y.end(), results.begin(), results.end());
       }
     }
   } else {
@@ -1013,7 +1009,7 @@ std::vector<std::vector<double>> SCPCM4GridOneDim(
       RcppThread::ProgressBar bar(unique_lib_sizes.size(), 1);
       RcppThread::parallelFor(0, unique_lib_sizes.size(), [&](size_t i) {
         int lib_size = unique_lib_sizes[i];
-        auto results = SCPCMSingle4GridOneDim(
+        local_results[i] = SCPCMSingle4GridOneDim(
           xEmbedings,
           yPred,
           zMatrixs,
@@ -1032,13 +1028,12 @@ std::vector<std::vector<double>> SCPCM4GridOneDim(
           parallel_level,
           cumulate
         );
-        x_xmap_y.insert(x_xmap_y.end(), results.begin(), results.end());
         bar++;
       }, threads_sizet);
     } else {
       RcppThread::parallelFor(0, unique_lib_sizes.size(), [&](size_t i) {
         int lib_size = unique_lib_sizes[i];
-        auto results = SCPCMSingle4GridOneDim(
+        local_results[i] = SCPCMSingle4GridOneDim(
           xEmbedings,
           yPred,
           zMatrixs,
@@ -1057,11 +1052,18 @@ std::vector<std::vector<double>> SCPCM4GridOneDim(
           parallel_level,
           cumulate
         );
-        x_xmap_y.insert(x_xmap_y.end(), results.begin(), results.end());
       }, threads_sizet);
     }
   }
 
+  // Initialize the result container
+  std::vector<PartialCorRes> x_xmap_y;
+
+  // Merge all local results into the final result
+  for (const auto& local_result : local_results) {
+    x_xmap_y.insert(x_xmap_y.end(), local_result.begin(), local_result.end());
+  }
+
   // Group by the first int and store second and third values as pairs
   std::map<int, std::vector<std::pair<double, double>>> grouped_results;
 
